
import curses, level, menu

statusline = \
  '"%s" _=floor #=wall $=cash .=goal @=plyr enter=menu (%d,%d)'

class Exit: pass

class Main:

	def __init__(s, args):
		s.level = level.Level()
		s.x, s.y = 5, 2 # ad hoc values, no real meaning
		s.savekey = curses.KEY_RIGHT
		s.message = None
		if len(args) > 0: s.load_level(args[0])

	def main(s, stdscr):
		s.stdscr = stdscr
		curses.curs_set( 1 )
		stdscr.leaveok( 0 )
		s.recenter_at_cursor()
		s.mainloop()

	def mainloop(s):
		e = 'Got an error %s.  Want to abort? [yes/no]'
		while(1):
			try:
				s.do_refresh()
				s.handle_input(s.stdscr.getch())
			except Exit: raise
			except:
				import sys
				t, val, trace = sys.exc_info()
				ans = menu.ask_input(s.stdscr, e % t)
				if ans == 'yes': raise
				s.recenter_at_cursor()

	def recenter_at_cursor(s):
		height, width = s.stdscr.getmaxyx()
		s.min_x = s.x - (width // 2)
		s.min_y = s.y - (height // 2) + 1
		s.max_x = s.min_x + width
		s.max_y = s.min_y + height - 1
		for y in range(s.min_y, s.max_y):
			for x in range(s.min_x, s.max_x):
				s.draw(x, y)

	def draw_status_bar(s):
		msg = s.message or statusline % (s.level.levelname, s.x, s.y)
		menu.write_line(s.stdscr, 0, msg)

	def do_refresh(s):
		s.draw_status_bar()
		s.stdscr.move(s.y - s.min_y + 1, s.x - s.min_x)
		s.stdscr.refresh()

	def draw(s, x, y):
		obj = s.level.get(x, y)
		if obj == '_': obj = ' '
		if x < s.min_x or y < s.min_y or \
		   x >= s.max_x or y >= s.max_y or \
		   (x == s.max_x - 1 and y == s.max_y - 1): return
		s.stdscr.addch(y - s.min_y + 1, x - s.min_x, obj)

	def handle_input(s, ch):
		s.message = ''
		try: c = chr(ch)
		except ValueError: c = '?'
		if ch in (curses.KEY_LEFT, curses.KEY_RIGHT,
			curses.KEY_UP, curses.KEY_DOWN):
			s.handle_arrow(ch)
		elif c in ' _-.$#@432,mn':
			s.handle_change(c)
		elif ch == 10: s.menu() # enter
		elif ch == 12: s.recenter_at_cursor() # ctrl-l
		else: curses.beep()

	def handle_arrow(s, ch):
		if ch == curses.KEY_LEFT: s.x = s.x - 1
		if ch == curses.KEY_RIGHT: s.x = s.x + 1
		if ch == curses.KEY_UP: s.y = s.y - 1
		if ch == curses.KEY_DOWN: s.y = s.y + 1
		s.savekey = ch
		if s.x < s.min_x or s.y < s.min_y or \
		   s.x >= s.max_x or s.y >= s.max_y: s.recenter_at_cursor()

	def handle_change(s, ch):
		if ch in ',4': ch = '$'
		if ch in 'm3': ch = '#'
		if ch in 'n2': ch = '@'
		if ch == '-': ch = '_'
		s.level.change(s.x, s.y, ch)
		for x, y in ((s.x, s.y), (s.x - 1, s.y), (s.x + 1, s.y),
				(s.x, s.y - 1), (s.x, s.y + 1)):
			s.draw(x, y)
		s.handle_arrow(s.savekey)

	def menu(s):
		opt = (('s', 'save level'), ('r', 'read/revert'),
			('l', 'level name'), ('c', 'change file'),
			('0', 'go to origin'),
			('q', 'quit'), ('e', 'exit menu'))
		opt2 = (('s', 'save w/ new name'), ('r', 'read another level'),
			('n', 'start new level'), ('e', 'exit menu'))
		scr = s.stdscr
		a = menu.ask_menu(scr, opt)
		if a == 'c':
			a = menu.ask_menu(scr, opt2)
			if a == 'n':
				s.level = level.Level()
				s.message = 'New level'
			elif a != 'e': s.level.ask_file(scr)
			# Warning: the second answer will propagate
		if a == 'q': raise Exit
		elif a == '0': s.x, s.y = 0, 0
		elif a == 'l': s.level.ask_name(scr)
		elif a == 'r':
			s.level.maybe_ask_file(scr)
			s.load_level()
		elif a == 's':
			s.level.maybe_ask_file(scr)
			s.level.maybe_ask_name(scr)
			s.message = s.level.save_level()
		elif a != 'e': curses.beep()
		s.recenter_at_cursor()

	def load_level(s, filename = None):
		retval = s.level.load_level(filename)
		if type(retval) is str: s.message = retval
		else: s.x, s.y = retval

